# Filmtipset.se metadata agent for Plex
# Adds Swedish titles, summaries, directors, actors, writers, genres, ratings from www.filmtipset.se to movies

import htmlentitydefs, re

BASE                = 'http://nyheter24.se/filmtipset/api/api.cgi'
API                 = 'k99kjZmkaBhTYt37171QcA'
USERNR              = Prefs['ftid']
FT_FROM_IMDB        = '%s?accesskey=%s&action=imdb&id=%%s&usernr=%s&nocomments=1&returntype=json' % (BASE, API, USERNR)
variants            = {'Jim Wynorski' : 'Wynorski', 'Russ Meyer' : 'Russ Meyer', 'Andy Sidaris': 'Sidaris', 'Christian Drew Sidaris' : 'Sidaris', 'Ingmar Bergman' : 'Ingmar Bergman','Russ Meyer':'Russ Meyer', 'Vilgot Sjöman': 'Vilgot','Fred Olen Ray':'Mr Ray'}
debugmode           = True

Log.Debug(FT_FROM_IMDB)

##
# Removes HTML or XML character references and entities from a text string.
# http://effbot.org/zone/re-sub.htm#unescape-html
#
# @param text The HTML (or XML) source text.
# @return The plain text, as a Unicode string, if necessary.

def unescape(text):
  def fixup(m):
    text = m.group(0)
    if text[:2] == "&#":
      # character reference
      try:
        if text[:3] == "&#x":
          return unichr(int(text[3:-1], 16))
        else:
          return unichr(int(text[2:-1]))
      except ValueError:
        pass
    else:
      # named entity
      try:
        text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
      except KeyError:
        pass
    return text # leave as is
  return re.sub("&#?\w+;", fixup, text)


# Just a function to easier debug the code in the console
def bugga(obj):
  if debugmode:
    Log.Debug('\nDBG: '+obj)

# Checks if the movie should be added to any collection depending on the director
def add2collection(director,titel):
  if director in variants:
    bugga('%s - COLL - %s' % (titel, variants[director]))
    return True
  else:
    bugga('%s - NOCOLL - %s' % (titel, director))
    return False


def Start():
  HTTP.CacheTime = CACHE_1DAY
  HTTP.Headers['User-Agent'] = 'Plex/Nine'


class Filmtipsetgent(Agent.Movies):
  name = 'Filmtipset'
  languages = ['sv']
  primary_provider = False
  contributes_to = ['com.plexapp.agents.imdb']
  
  def search(self, results, media, lang):
    # Use the IMDB id found by the primary metadata agent (IMDB/Freebase)
    results.Append(MetadataSearchResult(id=media.primary_metadata.id, score=100))
  
  def update(self, metadata, media, lang):
    # Only use data from Filmtipset if the user has set the language for this section to Swedish (Svenska)
    if lang == 'sv':
      try:
        # Create movie object from URL
        movie = JSON.ObjectFromURL(FT_FROM_IMDB % (metadata.id.lstrip('t')), encoding="ISO-8859-1")[0]['data'][0]['movie']

        # Title
        metadata.title = ''
        if Prefs['title']:
          metadata.title = movie['name'].encode('ISO-8859-1')
          metadata.original_title = movie['orgname'].encode('ISO-8859-1')
          bugga('FILMTITELN: \t'+metadata.title)
        
        # Year
        if movie['year'] and str(movie['year']).strip() != '':
          metadata.year = int(movie['year'])
          bugga('År: %s' % (metadata.year))

        # Summary
        metadata.summary = ''
        if Prefs['summary']:
          metadata.summary = movie['description'].encode('ISO-8859-1')
          bugga('Summary: '+metadata.summary)
        
        # Directors.
        if Prefs['director']:
          metadata.directors.clear()
          metadata.collections.clear()
          if movie['directorids']:
            for regi in movie['directorids']:
              r = regi['director']['name'].encode('ISO-8859-1')
              bugga('Regissör: '+r)
              metadata.directors.add(r)
              if add2collection(r, metadata.title):
                metadata.collections.add(variants[r])
        
        # Writers.
        if Prefs['writer']:
          if movie['writer'].strip() != '':
            metadata.writers.clear()
            if movie['writerids']:
              for writ in movie['writerids']:
                metadata.writers.add(writ['writer']['name'].encode('ISO-8859-1'))
                bugga('Manus: '+writ['writer']['name'].encode('ISO-8859-1'))
     
        # Actors.
        metadata.roles.clear()
        if movie['actorids']:
          for movie_role in movie['actorids']:
            role = metadata.roles.new()
            role.actor = movie_role['actor']['name'].encode('ISO-8859-1')
            bugga('Skådis: '+role.actor)
        
        # Rating
        metadata.rating = None
        if Prefs['rating']:
          if movie['grade']:
            bugga('GRADE FINNS')
            if movie['grade']['value']:
              bugga('Betyg: '+movie['grade']['value'])
              metadata.rating = float(movie['grade']['value'])*2
            else:
              bugga('Betyg: Problem med betyg')
          elif movie['filmtipsetgrade']:
            if movie['filmtipsetgrade']['value']:
              bugga('FT-betyg: '+movie['filmtipsetgrade']['value'])
              metadata.rating = float(movie['filmtipsetgrade']['value'])*2
            else:
              bugga('FT-Betyg: Problem med FT-betyg')
        
        # Genres
        metadata.genres.clear()
        if Prefs['genre']:
          genres = movie['genres']
          if genres:
            metadata.genres.clear()
            for val in genres:
              bugga('Genre: '+unescape(val['genre']))
              metadata.genres.add(unescape(val['genre']))
        
        # Countrys
        metadata.countries.clear()
        if movie['country']:
          for x in movie['country'].split(','):
            bugga('Land: '+x.strip().encode('ISO-8859-1'))
            metadata.countries.add(x.strip().encode('ISO-8859-1'))
        else:
          bugga('Land: inga länder')      
        
        # Get the poster from Moviepilot if it is available
        try:
          poster_url = movie['image']
          bugga('Poster: '+poster_url)
          if poster_url == 'http://images.filmtipset.se/posters/NONE':
            poster_url = None
        except:
          bugga('Poster: Inga')
          poster_url = None

        if poster_url is not None:
          if Prefs['poster']:
            try:
              if poster_url not in metadata.posters:
                img = HTTP.Request(poster_url)
                metadata.posters[poster_url] = Proxy.Preview(img)
            except:
              pass
          else:
            del metadata.posters[poster_url]
      
      except:
        pass
